package org.fuseri.moduleexercise.answer;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.fuseri.model.dto.exercise.AnswerDto;
import org.fuseri.model.dto.exercise.AnswerInQuestionCreateDto;
import org.fuseri.model.dto.exercise.AnswersCreateDto;
import org.fuseri.model.dto.exercise.ExerciseCreateDto;
import org.fuseri.model.dto.exercise.ExerciseDto;
import org.fuseri.model.dto.exercise.QuestionCreateDto;
import org.fuseri.model.dto.exercise.QuestionDto;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.http.MediaType;
import org.springframework.test.web.servlet.MockMvc;
import java.util.List;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.post;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.put;

@SpringBootTest
@AutoConfigureMockMvc
public class AnswerTest {

    @Autowired
    ObjectMapper objectMapper;
    @Autowired
    private MockMvc mockMvc;

    public static String asJsonString(final Object obj) {
        try {
            return new ObjectMapper().writeValueAsString(obj);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    private QuestionDto createQuestion(long id) throws Exception {
        var question = new QuestionCreateDto("this statement is false", id,
                List.of(new AnswerInQuestionCreateDto("dis a logical paradox", true)));


        var posted = mockMvc.perform(post("/questions")
                .content(asJsonString(question))
                .contentType(MediaType.APPLICATION_JSON));

        var cont = posted.andReturn().getResponse().getContentAsString();
        var res = objectMapper.readValue(cont, QuestionDto.class);
        return res;
    }


    private long createExercise() {
        var postExercise = new ExerciseCreateDto("idioms", "exercise on basic idioms", 2, 0);

        long id = 0L;

        try {
            var dis = mockMvc.perform(post("/exercises")
                    .content(asJsonString(postExercise))
                    .contentType(MediaType.APPLICATION_JSON));
            var ok = dis.andReturn().getResponse().getContentAsString();

            var ll = objectMapper.readValue(ok, ExerciseDto.class);

            id = ll.getId();
        } catch (Exception e) {
            assert (false);
        }
        return id;
    }


    @Test
    void testCreateAnswer() throws Exception {

        List<AnswerDto> res = createAnswer();

        var expected1 = new AnswerDto("True", false);
        var expected2 = new AnswerDto("False", false);

        assert (res.get(0).equals(expected1));
        assert (res.get(1).equals(expected2));

    }

    private List<AnswerDto> createAnswer() throws Exception {
        var exerciseId = createExercise();
        var question = createQuestion(exerciseId);

        var incorrect1 = new AnswerInQuestionCreateDto("True", false);
        var incorrect2 = new AnswerInQuestionCreateDto("False", false);

        var createAnswer = new AnswersCreateDto(question.getId(), List.of(incorrect1, incorrect2));

        var posted = mockMvc.perform(post("/answers")
                .content(asJsonString(createAnswer))
                .contentType(MediaType.APPLICATION_JSON));

        var asStr = posted.andReturn().getResponse().getContentAsString();

        var res = objectMapper.readValue(asStr, new TypeReference<List<AnswerDto>>() {
        });
        return res;
    }

    @Test
    void testUpdate() throws Exception {

        var exerciseId = createExercise();
        var question = createQuestion(exerciseId);

        var incorrect1 = new AnswerInQuestionCreateDto("True", false);
        var incorrect2 = new AnswerInQuestionCreateDto("False", false);


        var createAnswer = new AnswersCreateDto(question.getId(), List.of(incorrect1, incorrect2));


        var posted = mockMvc.perform(post("/answers")
                .content(asJsonString(createAnswer))
                .contentType(MediaType.APPLICATION_JSON));

        var asStr = posted.andReturn().getResponse().getContentAsString();

        var res = objectMapper.readValue(asStr, new TypeReference<List<AnswerDto>>() {
        });


        var updated = """
                {
                  "text": "dis true",
                  "correct": false,
                  "questionId": "%s"
                }
                """;


        updated = String.format(updated, question.getId());

        var puts = mockMvc.perform(put(String.format("/answers/%s", res.get(0).getId()))
                .content(updated).contentType(MediaType.APPLICATION_JSON));

        var content = puts.andReturn().getResponse().getContentAsString();

        var res2 = objectMapper.readValue(content, AnswerDto.class);

        var expected = new AnswerDto("dis true", false);

        assert res2.equals(expected);

    }

}
